@@ -33,6 +33,13 @@ using namespace GPSMap60CSx;
 using namespace Garmin;
 using namespace std;
 
+#if defined(HAVE_BIGENDIAN) || !defined(CAN_UNALIGNED)
+#  define DBG_SHOW_WAYPOINT
+#  define UNTESTED throw exce_t(errSync, "This function has not yet been tested on your platform.")
+#else
+#  define UNTESTED
+#endif
+
 namespace GPSMap60CSx
 {
 
@@ -122,7 +129,7 @@ namespace GPSMap60CSx
             command.type = GUSB_APPLICATION_LAYER;
             command.id   = Pid_Command_Data;
             command.size = 2;
-            *(uint16_t*)command.payload = Cmnd_Start_Pvt_Data;
+            *(uint16_t*)command.payload = gar_endian(uint16_t, Cmnd_Start_Pvt_Data);
             dev->usb->write(command);
 
             while(dev->doRealtimeThread) {
@@ -143,7 +150,7 @@ namespace GPSMap60CSx
             command.type = GUSB_APPLICATION_LAYER;
             command.id   = Pid_Command_Data;
             command.size = 2;
-            *(uint16_t*)command.payload = Cmnd_Stop_Pvt_Data;
+            *(uint16_t*)command.payload = gar_endian(uint16_t, Cmnd_Stop_Pvt_Data);
             dev->usb->write(command);
 
             dev->_release();
@@ -164,6 +171,8 @@ namespace GPSMap60CSx
 
 CDevice::CDevice()
 : devid(0)
+, screenhflip(false)
+, screenvflip(false)
 , usb(0)
 , doRealtimeThread(false)
 , pScreen(0)
@@ -194,9 +203,6 @@ const string& CDevice::getCopyright()
 
 void CDevice::_acquire()
 {
-#if defined(WORDS_BIGENDIAN) || !defined(CAN_UNALIGNED)
-    throw exce_t(errSync, "This device has not yet been ported to your platform.");
-#endif
     usb = new CUSB();
     usb->open();
     if(devid == 0x01a5) {
@@ -220,6 +226,7 @@ void CDevice::_acquire()
 void CDevice::_uploadMap(const uint8_t * mapdata, uint32_t size, const char * key)
 {
     if(usb == 0) return;
+    UNTESTED;
     Packet_t command;
     Packet_t response;
     int cancel = 0;
@@ -235,16 +242,16 @@ void CDevice::_uploadMap(const uint8_t * mapdata, uint32_t size, const char * ke
     command.type = GUSB_APPLICATION_LAYER;
     command.id   = Pid_Command_Data;
     command.size = 2;
-    *(uint16_t*)command.payload = Cmnd_Transfer_Mem;
+    *(uint16_t*)command.payload = gar_endian(uint16_t, Cmnd_Transfer_Mem);
     usb->write(command);
 
     while(usb->read(response)) {
         if(response.id == Pid_Capacity_Data) {
-            cout << "free memory: " << dec << (((uint32_t*)response.payload)[1] / (1024*1024)) << " MB" << endl;
-            uint32_t memory = ((uint32_t*)response.payload)[1];
+            uint32_t memory = gar_ptr_load(uint32_t, response.payload + 4);
+            cout << "free memory: " << dec << memory / (1024*1024) << " MB" << endl;
             if(memory < size) {
                 stringstream msg;
-                msg << "Failed to send map: Unit has not enought memory (available/needed): " << memory << "/" << size << " bytes";
+                msg << "Failed to send map: Unit has not enough memory (available/needed): " << memory << "/" << size << " bytes";
                 throw exce_t(errRuntime,msg.str());
             }
         }
@@ -271,7 +278,7 @@ void CDevice::_uploadMap(const uint8_t * mapdata, uint32_t size, const char * ke
     command.type = GUSB_APPLICATION_LAYER;
     command.id   = 75;
     command.size = 2;
-    *(uint16_t*)command.payload = 0x000A;
+    *(uint16_t*)command.payload = gar_endian(uint16_t, 0x000A);
     usb->write(command);
 
     while(usb->read(response)) {
@@ -291,7 +298,7 @@ void CDevice::_uploadMap(const uint8_t * mapdata, uint32_t size, const char * ke
         chunkSize       = (size < (GUSB_PAYLOAD_SIZE - sizeof(offset))) ? size : (GUSB_PAYLOAD_SIZE - sizeof(offset));
         command.size    = chunkSize + sizeof(offset);
 
-        *(uint32_t*)command.payload = offset;
+        *(uint32_t*)command.payload = gar_endian(uint32_t, offset);
         memcpy(command.payload + sizeof(offset),mapdata,chunkSize);
         size    -= chunkSize;
         mapdata += chunkSize;
@@ -300,7 +307,7 @@ void CDevice::_uploadMap(const uint8_t * mapdata, uint32_t size, const char * ke
         usb->write(command);
 
         double progress = ((total - size) * 100.0) / total;
-        callback(progress,0,&cancel,0,"Transfering map data.");
+        callback(progress,0,&cancel,0,"Transferring map data.");
     }
 
     callback(100,0,&cancel,0,"done");
@@ -309,7 +316,7 @@ void CDevice::_uploadMap(const uint8_t * mapdata, uint32_t size, const char * ke
     command.type = GUSB_APPLICATION_LAYER;
     command.id   = 45;
     command.size = 2;
-    *(uint16_t*)command.payload = 0x000A;
+    *(uint16_t*)command.payload = gar_endian(uint16_t, 0x000A);
     usb->write(command);
 }
 
@@ -317,6 +324,7 @@ void CDevice::_uploadMap(const uint8_t * mapdata, uint32_t size, const char * ke
 void CDevice::_uploadMap(const char * filename, uint32_t size, const char * key)
 {
     if(usb == 0) return;
+    UNTESTED;
     Packet_t command;
     Packet_t response;
     int cancel = 0;
@@ -332,16 +340,16 @@ void CDevice::_uploadMap(const char * filename, uint32_t size, const char * key)
     command.type = GUSB_APPLICATION_LAYER;
     command.id   = Pid_Command_Data;
     command.size = 2;
-    *(uint16_t*)command.payload = Cmnd_Transfer_Mem;
+    *(uint16_t*)command.payload = gar_endian(uint16_t, Cmnd_Transfer_Mem);
     usb->write(command);
 
     while(usb->read(response)) {
         if(response.id == Pid_Capacity_Data) {
-            cout << "free memory: " << dec << (((uint32_t*)response.payload)[1] / (1024*1024)) << " MB" << endl;
-            uint32_t memory = ((uint32_t*)response.payload)[1];
+            uint32_t memory = gar_ptr_load(uint32_t, response.payload + 4);
+            cout << "free memory: " << dec << memory / (1024*1024) << " MB" << endl;
             if(memory < size) {
                 stringstream msg;
-                msg << "Failed to send map: Unit has not enought memory (available/needed): " << memory << "/" << size << " bytes";
+                msg << "Failed to send map: Unit has not enough memory (available/needed): " << memory << "/" << size << " bytes";
                 throw exce_t(errRuntime,msg.str());
             }
         }
@@ -368,7 +376,7 @@ void CDevice::_uploadMap(const char * filename, uint32_t size, const char * key)
     command.type = GUSB_APPLICATION_LAYER;
     command.id   = 75;
     command.size = 2;
-    *(uint16_t*)command.payload = 0x000A;
+    *(uint16_t*)command.payload = gar_endian(uint16_t, 0x000A);
     usb->write(command);
 
     while(usb->read(response)) {
@@ -399,7 +407,7 @@ void CDevice::_uploadMap(const char * filename, uint32_t size, const char * key)
 
         fread(buffer, chunkSize, 1, fid);
 
-        *(uint32_t*)command.payload = offset;
+        *(uint32_t*)command.payload = gar_endian(uint32_t, offset);
         memcpy(command.payload + sizeof(offset),buffer,chunkSize);
         size    -= chunkSize;
         offset  += chunkSize;
@@ -407,7 +415,7 @@ void CDevice::_uploadMap(const char * filename, uint32_t size, const char * key)
         usb->write(command);
 
         double progress = ((total - size) * 100.0) / total;
-        callback(progress,0,&cancel,0,"Transfering map data.");
+        callback(progress,0,&cancel,0,"Transferring map data.");
     }
 
     callback(100,0,&cancel,0,"done");
@@ -416,7 +424,7 @@ void CDevice::_uploadMap(const char * filename, uint32_t size, const char * key)
     command.type = GUSB_APPLICATION_LAYER;
     command.id   = 45;
     command.size = 2;
-    *(uint16_t*)command.payload = 0x000A;
+    *(uint16_t*)command.payload = gar_endian(uint16_t, 0x000A);
     usb->write(command);
 }
 
@@ -425,6 +433,7 @@ void CDevice::_queryMap(std::list<Map_t>& maps)
 {
     maps.clear();
     if(usb == 0) return;
+    UNTESTED;
 
     Packet_t command;
     Packet_t response;
@@ -509,7 +518,7 @@ void CDevice::_downloadWaypoints(list<Garmin::Wpt_t>& waypoints)
     command.type = GUSB_APPLICATION_LAYER;
     command.id   = Pid_Command_Data;
     command.size = 2;
-    *(uint16_t*)command.payload = Cmnd_Transfer_Wpt;
+    *(uint16_t*)command.payload = gar_endian(uint16_t, Cmnd_Transfer_Wpt);
     usb->write(command);
 
     while(1) {
@@ -517,7 +526,7 @@ void CDevice::_downloadWaypoints(list<Garmin::Wpt_t>& waypoints)
 
         if(response.id == Pid_Records) {
 #ifdef DBG_SHOW_WAYPOINT
-            cout << "number of waypoints:" << *(int16_t*)response.payload << endl;
+            cout << "number of waypoints:" << gar_ptr_load(uint16_t, response.payload) << endl;
 #endif
         }
 
@@ -539,7 +548,7 @@ void CDevice::_downloadWaypoints(list<Garmin::Wpt_t>& waypoints)
     command.type = GUSB_APPLICATION_LAYER;
     command.id   = Pid_Command_Data;
     command.size = 2;
-    *(uint16_t*)command.payload = Cmnd_Transfer_Prx;
+    *(uint16_t*)command.payload = gar_endian(uint16_t, Cmnd_Transfer_Prx);
     usb->write(command);
 
     while(1) {
@@ -549,7 +558,7 @@ void CDevice::_downloadWaypoints(list<Garmin::Wpt_t>& waypoints)
         if(response.id == Pid_Records) {
             //TODO read data
 #ifdef DBG_SHOW_WAYPOINT
-            cout << "number of proximity waypoints:" << *(int16_t*)response.payload << endl;
+            cout << "number of proximity waypoints:" << gar_ptr_load(uint16_t, response.payload) << endl;
 #endif
         }
 
@@ -627,7 +636,7 @@ void CDevice::_uploadWaypoints(std::list<Garmin::Wpt_t>& waypoints)
         command.type = GUSB_APPLICATION_LAYER;
         command.id   = Pid_Records;
         command.size = 2;
-        *(uint16_t*)command.payload = prx_wpt_cnt;
+        *(uint16_t*)command.payload = gar_endian(uint16_t, prx_wpt_cnt);
         usb->write(command);
 
         wpt = waypoints.begin();
@@ -649,7 +658,7 @@ void CDevice::_uploadWaypoints(std::list<Garmin::Wpt_t>& waypoints)
         command.type = GUSB_APPLICATION_LAYER;
         command.id   = Pid_Xfer_Cmplt;
         command.size = 2;
-        *(uint16_t*)command.payload = Cmnd_Transfer_Prx;
+        *(uint16_t*)command.payload = gar_endian(uint16_t, Cmnd_Transfer_Prx);
         usb->write(command);
 
     }
@@ -659,7 +668,7 @@ void CDevice::_uploadWaypoints(std::list<Garmin::Wpt_t>& waypoints)
     command.type = GUSB_APPLICATION_LAYER;
     command.id   = Pid_Records;
     command.size = 2;
-    *(uint16_t*)command.payload = waypoints.size();
+    *(uint16_t*)command.payload = gar_endian(uint16_t, waypoints.size());
     usb->write(command);
 
     wpt = waypoints.begin();
@@ -680,7 +689,7 @@ void CDevice::_uploadWaypoints(std::list<Garmin::Wpt_t>& waypoints)
     command.type = GUSB_APPLICATION_LAYER;
     command.id   = Pid_Xfer_Cmplt;
     command.size = 2;
-    *(uint16_t*)command.payload = Cmnd_Transfer_Wpt;
+    *(uint16_t*)command.payload = gar_endian(uint16_t, Cmnd_Transfer_Wpt);
     usb->write(command);
 
 }
@@ -704,13 +713,18 @@ void CDevice::_downloadTracks(std::list<Garmin::Track_t>& tracks)
     command.type = GUSB_APPLICATION_LAYER;
     command.id   = Pid_Command_Data;
     command.size = 2;
-    *(uint16_t*)command.payload = Cmnd_Transfer_Trk;
+    *(uint16_t*)command.payload = gar_endian(uint16_t, Cmnd_Transfer_Trk);
     usb->write(command);
 
     int         trackidx = 0;
     string      name;
     Track_t *   track = 0;
-    while(1) {
+    int         cancel = 0;
+    int         npts = 0;
+    int         ntotal = 65535;
+    callback(0,0,&cancel,"Download tracks ...",0);
+
+    while(!cancel) {
 
         if(!usb->read(response)) continue;
 
@@ -725,6 +739,10 @@ void CDevice::_downloadTracks(std::list<Garmin::Track_t>& tracks)
 
         }
 
+        if(response.id == Pid_Records) {
+            ntotal = gar_ptr_load(uint16_t, response.payload);
+        }
+
         if(response.id == Pid_Trk_Data) {
             D302_Trk_t * data = (D302_Trk_t*)response.payload;
             TrkPt_t pt;
@@ -746,12 +764,26 @@ void CDevice::_downloadTracks(std::list<Garmin::Track_t>& tracks)
 
             pt << *data;
             track->track.push_back(pt);
+
+            if (++npts % 100 == 0) {
+                double progress = (npts * 100.0) / ntotal;
+                callback(progress,0,&cancel,0,"Transferring track data.");
+            }
         }
 
         if(response.id == Pid_Xfer_Cmplt) {
             break;
         }
     }
+    if (cancel) {
+        command.type = GUSB_APPLICATION_LAYER;
+        command.id   = Pid_Command_Data;
+        command.size = 2;
+        *(uint16_t*)command.payload = gar_endian(uint16_t, Cmnd_Abort_Transfer);
+        usb->write(command);
+    }
+
+    callback(100,0,&cancel,0,"done");
 }
 
 
@@ -779,7 +811,7 @@ void CDevice::_uploadTracks(std::list<Garmin::Track_t>& tracks)
         command.type = GUSB_APPLICATION_LAYER;
         command.id   = Pid_Records;
         command.size = 2;
-        *(uint16_t*)command.payload = nrec;
+        *(uint16_t*)command.payload = gar_endian(uint16_t, nrec);
         usb->write(command);
 
         // write track header
@@ -821,7 +853,7 @@ void CDevice::_uploadTracks(std::list<Garmin::Track_t>& tracks)
         command.type = GUSB_APPLICATION_LAYER;
         command.id   = Pid_Xfer_Cmplt;
         command.size = 2;
-        *(uint16_t*)command.payload = Cmnd_Transfer_Trk;
+        *(uint16_t*)command.payload = gar_endian(uint16_t, Cmnd_Transfer_Trk);
         usb->write(command);
         ++track;
     }
@@ -829,6 +861,87 @@ void CDevice::_uploadTracks(std::list<Garmin::Track_t>& tracks)
 }
 
 
+void CDevice::_downloadRoutes(std::list<Garmin::Route_t>& routes)
+{
+    routes.clear();
+    if(usb == 0) return;
+
+    Packet_t command;
+    Packet_t response;
+
+    // ???
+    command.type = GUSB_APPLICATION_LAYER;
+    command.id   = 0x1C;
+    command.size = 2;
+    *(uint16_t*)command.payload = 0x0000;
+    usb->write(command);
+
+    command.type = GUSB_APPLICATION_LAYER;
+    command.id   = Pid_Command_Data;
+    command.size = 2;
+    *(uint16_t*)command.payload = gar_endian(uint16_t, Cmnd_Transfer_Rte);
+    usb->write(command);
+
+    int         routeidx = 0;
+    string      name;
+    Route_t *   route = 0;
+    int         cancel = 0;
+    int         npts = 0;
+    int         ntotal = 65535;
+    callback(0,0,&cancel,"Download routes ...",0);
+
+    while(!cancel) {
+
+        if(!usb->read(response)) continue;
+
+        if(response.id == Pid_Rte_Hdr) {
+            routeidx = 0;
+            D202_Rte_Hdr_t * hdr = (D202_Rte_Hdr_t*)response.payload;
+            routes.push_back(Route_t());
+            route = &routes.back();
+
+            *route << *hdr;
+            name  = hdr->ident;
+        }
+
+        if(response.id == Pid_Records) {
+            ntotal = gar_ptr_load(uint16_t, response.payload);
+        }
+
+        if(response.id == Pid_Rte_Wpt_Data) {
+            D110_Wpt_t * p = (D110_Wpt_t*)response.payload;
+            route->route.push_back(RtePt_t());
+            RtePt_t& rtept = route->route.back();
+            rtept << *p;
+
+            if (++npts % 50 == 0) {
+                double progress = (npts * 100.0) / ntotal;
+                callback(progress,0,&cancel,0,"Transferring route data.");
+            }
+        }
+
+        if(response.id == Pid_Rte_Link_Data) {
+            D210_Rte_Link_t * l = (D210_Rte_Link_t*)response.payload;
+            RtePt_t& rtept = route->route.back();
+            rtept << *l;
+        }
+
+        if(response.id == Pid_Xfer_Cmplt) {
+            break;
+        }
+    }
+    if (cancel) {
+        command.type = GUSB_APPLICATION_LAYER;
+        command.id   = Pid_Command_Data;
+        command.size = 2;
+        *(uint16_t*)command.payload = gar_endian(uint16_t, Cmnd_Abort_Transfer);
+        usb->write(command);
+    }
+
+    callback(100,0,&cancel,0,"done");
+}
+
+
 void CDevice::_uploadRoutes(list<Garmin::Route_t>& routes)
 {
     if(usb == 0) return;
@@ -850,12 +963,12 @@ void CDevice::_uploadRoutes(list<Garmin::Route_t>& routes)
     list<Garmin::Route_t>::const_iterator route = routes.begin();
     while(route != routes.end()) {
         //announce number of records
-        // D202_Rte_Hdr_t + (D110_Wpt_t + D210_Tre_Link_t) * number of route points
+        // D202_Rte_Hdr_t + (D110_Wpt_t + D210_Rte_Link_t) * number of route points
         uint16_t nrec = 1 + route->route.size() * 2;
         command.type = GUSB_APPLICATION_LAYER;
         command.id   = Pid_Records;
         command.size = 2;
-        *(uint16_t*)command.payload = nrec;
+        *(uint16_t*)command.payload = gar_endian(uint16_t, nrec);
         usb->write(command);
 
         // write route header
@@ -896,7 +1009,7 @@ void CDevice::_uploadRoutes(list<Garmin::Route_t>& routes)
         command.type = GUSB_APPLICATION_LAYER;
         command.id   = Pid_Xfer_Cmplt;
         command.size = 2;
-        *(uint16_t*)command.payload = Cmnd_Transfer_Rte;
+        *(uint16_t*)command.payload = gar_endian(uint16_t, Cmnd_Transfer_Rte);
         usb->write(command);
         ++route;
     }
@@ -906,7 +1019,9 @@ void CDevice::_uploadRoutes(list<Garmin::Route_t>& routes)
 
 void CDevice::_uploadCustomIcons(list<Garmin::Icon_t>& icons)
 {
-    if(usb == 0) return;
+    cout << "running uploadCustomIcons for device " << hex << devid << endl;
+
+	if(usb == 0) return;
 
     if(devid == 0x0231) return IDeviceDefault::_uploadCustomIcons(icons);
 
@@ -928,7 +1043,7 @@ void CDevice::_uploadCustomIcons(list<Garmin::Icon_t>& icons)
         command.type = GUSB_APPLICATION_LAYER;
         command.id   = Pid_Req_Icon_Id;
         command.size = 2;
-        *(uint16_t*)command.payload = icon->idx + 1;
+        *(uint16_t*)command.payload = gar_endian(uint16_t, icon->idx + 1);
         usb->write(command);
 
         while(usb->read(response)) {
@@ -947,6 +1062,7 @@ void CDevice::_uploadCustomIcons(list<Garmin::Icon_t>& icons)
         while(usb->read(response)) {
             if(response.id == Pid_Ack_Clr_Tbl) {
                 // send back color table
+                memcpy(response.payload + 4, icon->clrtbl, sizeof(icon->clrtbl));
                 command = response;
             }
         }
@@ -1069,19 +1185,38 @@ void CDevice::_screenshot(char *& clrtbl, char *& data, int& width, int& height)
     *(uint32_t*)command.payload = tan;
     usb->write(command);
 
-    if(devid == 0x0312 || devid == 0x02b6) {
-        for(int r = 0; r < screenheight; ++r) {
-            for(int c = 0; c < screenwidth; ++c) {
-                pScreen[r * screenwidth + c] = buffer[(r + 1) * screenwidth - c - 1];
+    cout << "device " << devname << " hor " << screenhflip << " vert " << screenvflip << endl;
+
+    if(screenhflip) {
+    	// screen shot is horizontally flipped
+    	if(screenvflip) {
+    		// screen shot is also vertically flipped
+            for(int r = 0; r < screenheight; ++r) {
+                for(int c = 0; c < screenwidth; ++c) {
+                    pScreen[r * screenwidth + c] = buffer[(screenheight - r) * screenwidth - c - 1];
+                }
             }
-        }
-    }
-    else {
-        for(int r = 0; r < screenheight; ++r) {
-            for(int c = 0; c < screenwidth; ++c) {
-                pScreen[r * screenwidth + c] = buffer[(screenheight - 1 - r) * screenwidth + c];
+    	} else {
+    		// vertical organisation of the screen shot is fine
+            for(int r = 0; r < screenheight; ++r) {
+                for(int c = 0; c < screenwidth; ++c) {
+                    pScreen[r * screenwidth + c] = buffer[(r + 1) * screenwidth - c - 1];
+                }
             }
-        }
+     	}
+    } else {
+    	// horizontal organisation of the screen shot is fine
+    	if(screenvflip) {
+    		// screen shot is vertically flipped
+    		for(int r = 0; r < screenheight; ++r) {
+    			memcpy(pScreen + r * screenwidth,
+    				   buffer + (screenheight - 1 - r) * screenwidth,
+    				   screenwidth);
+    		}
+    	} else {
+    		// vertical organisation of the screen shot is fine
+    		memcpy(pScreen, buffer, screenheight * screenwidth);
+    	}
     }
 
     clrtbl  = aClrtbl;
@@ -1135,7 +1270,7 @@ void CDevice::_getDevProperties(Garmin::DevProperties_t& dev_properties)
     command.type = GUSB_APPLICATION_LAYER;
     command.id   = Pid_Command_Data;
     command.size = 2;
-    *(uint16_t*)command.payload = Cmnd_Transfer_Mem;
+    *(uint16_t*)command.payload = gar_endian(uint16_t, Cmnd_Transfer_Mem);
     usb->write(command);
 
     // try to read SD Ram capacity
@@ -1143,8 +1278,8 @@ void CDevice::_getDevProperties(Garmin::DevProperties_t& dev_properties)
     uint16_t tile_limit = 0;
     while(usb->read(response)) {
         if(response.id == Pid_Capacity_Data) {
-            tile_limit = ((uint16_t*)response.payload)[1];
-            memory = ((uint32_t*)response.payload)[1];
+            tile_limit = gar_ptr_load(uint16_t, response.payload + 2);
+            memory = gar_ptr_load(uint32_t, response.payload + 4);
         }
     }
     if(tile_limit == 0) {
